#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
package ${package}.${artifactId};

import ${package}.keymgr.LicensingSchema;
import javax.annotation.concurrent.Immutable;
import javax.ws.rs.*;
import static javax.ws.rs.core.MediaType.*;
import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;
import net.java.truelicense.core.*;
import net.java.truelicense.core.io.*;
import net.java.truelicense.obfuscate.Obfuscate;

/**
 * A license consumer manager with a RESTful web service API.
 *
 * @author Christian Schlichtherle
 */
@Path("license")
@Produces({ APPLICATION_JSON, APPLICATION_XML, TEXT_XML })
@Immutable
public final class LicenseApi implements LicenseConsumerManager {

    @Obfuscate private static final String SUBJECT = "subject";

    private static final QName subject = new QName(SUBJECT);

    private static final LicenseConsumerManager cm = LicensingSchema.manager();

    @GET
    @Path("subject")
    @Produces(APPLICATION_JSON)
    public String subjectAsJson() { return '"' + subject() + '"'; }

    @GET
    @Path("subject")
    @Produces({ APPLICATION_XML, TEXT_XML })
    public JAXBElement<String> subjectAsXml() {
        return new JAXBElement<String>(subject, String.class, subject());
    }

    @GET
    @Path("subject")
    @Produces(TEXT_PLAIN)
    @Override
    public String subject() { return cm.subject(); }

    // Publishing this method via a web service API does not make sense.
    @Override
    public LicenseParameters parameters() { return cm.parameters(); }

    // Publishing this method via a web service API does not make sense.
    @Override
    public LicenseConsumerContext context() { return cm.context(); }

    @POST
    public License install(final byte[] key) throws LicenseManagementException {
        final MemoryStore ms = new MemoryStore();
        ms.data(key);
        return install(ms);
    }

    @Override
    public License install(Source source) throws LicenseManagementException {
        return cm.install(source);
    }

    @GET
    @Override
    public License view() throws LicenseManagementException {
        return cm.view();
    }

    // Publishing this method via a web service API does not make sense.
    @Override
    public void verify() throws LicenseManagementException {
        cm.verify();
    }

    @DELETE
    @Override
    public void uninstall() throws LicenseManagementException {
        cm.uninstall();
    }
}
